"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// packages/@code-like-a-carpenter/foundation-parser/src/extractors/cdc.ts
var cdc_exports = {};
__export(cdc_exports, {
  extractChangeDataCaptureConfig: () => extractChangeDataCaptureConfig,
  extractDispatcherConfig: () => extractDispatcherConfig,
  getTargetTables: () => getTargetTables
});
module.exports = __toCommonJS(cdc_exports);
var import_assert = __toESM(require("assert"));
var import_crypto = __toESM(require("crypto"));
var import_path = __toESM(require("path"));
var import_graphql = require("graphql");
var import_camelCase = __toESM(require("lodash/camelCase.js"));
var import_kebabCase = __toESM(require("lodash/kebabCase.js"));
var import_snakeCase = __toESM(require("lodash/snakeCase.js"));
var import_upperFirst = __toESM(require("lodash/upperFirst.js"));
var import_config = require("../config.cjs");
var import_helpers = require("../helpers.cjs");
var import_paths = require("../paths.cjs");
var import_tables = require("../tables.cjs");
function extractDispatcherConfig(config, outputFile, type) {
  const tableName = (0, import_tables.extractTableName)(type);
  const filename = `dispatcher-${(0, import_kebabCase.default)(tableName)}`;
  const functionName = `${tableName}CDCDispatcher`;
  const directory = import_path.default.join(import_path.default.dirname(outputFile), filename);
  (0, import_assert.default)(
    type?.astNode?.directives,
    `Expected to find at least one directive on type ${type.name}`
  );
  const { batchSize, maximumRetryAttempts, memorySize, timeout } = type.astNode.directives.filter((d) => d.name.value === "reacts" || d.name.value === "enriches").map(
    (directive) => import_config.DispatcherConfigSchema.parse({
      ...config.dispatcherDefaults,
      ...(0, import_helpers.getOptionalArgObjectValue)("dispatcherConfig", directive)
    })
  ).reduce((acc, next) => {
    return {
      batchSize: Math.max(acc.batchSize, next.batchSize),
      maximumRetryAttempts: Math.max(
        acc.maximumRetryAttempts,
        next.maximumRetryAttempts
      ),
      memorySize: Math.max(acc.memorySize, next.memorySize),
      timeout: Math.max(acc.timeout, next.timeout)
    };
  }, config.dispatcherDefaults);
  const nestedStackLocation = "./dispatcher.yml";
  return {
    batchSize,
    dependenciesModuleId: (0, import_paths.resolveDependenciesModuleId)(config, directory),
    directory,
    filename,
    functionName,
    maximumRetryAttempts,
    memorySize,
    nestedStackLocation,
    runtimeModuleId: "@code-like-a-carpenter/foundation-runtime",
    timeout
  };
}
function extractChangeDataCaptureConfig(config, schema, type, outputFile) {
  return type.astNode?.directives?.map((directive) => {
    if (directive.name.value === "enriches") {
      return extractEnricherConfig(
        config,
        schema,
        type,
        directive,
        outputFile
      );
    }
    if (directive.name.value === "reacts") {
      return extractReactorConfig(
        config,
        schema,
        type,
        directive,
        outputFile
      );
    }
    if (directive.name.value === "reduces") {
      return extractReducerConfig(
        config,
        schema,
        type,
        directive,
        outputFile
      );
    }
    return null;
  }).filter(import_helpers.filterNull) ?? [];
}
function getTargetTable(schema, modelName, produces) {
  const targetModel = schema.getType(produces);
  (0, import_assert.default)(
    targetModel,
    `\`targetModel\` arg on CDC directive for ${modelName} identifies ${produces}, which does not appear to identify a type`
  );
  return (0, import_tables.extractTableName)((0, import_graphql.assertObjectType)(targetModel));
}
function getTargetTables(fieldName, schema, directive) {
  const arg = (0, import_helpers.getOptionalArg)(fieldName, directive);
  if (!arg) {
    return [];
  }
  (0, import_assert.default)(arg.value.kind === "ListValue", `Expected ${fieldName} to be a list`);
  return Array.from(
    new Set(
      arg.value.values.map((v) => {
        (0, import_assert.default)(
          v.kind === "StringValue",
          `Expected @${directive.name.value} directive argument "${fieldName}" to be a list of strings`
        );
        return getTargetTable(schema, directive.name.value, v.value);
      })
    )
  );
}
function extractCommonConfig(config, schema, type, directive, outputFile, filename) {
  const sourceModelName = type.name;
  const handlerImportName = (0, import_helpers.getArgStringValue)("importName", directive);
  const handler = (0, import_helpers.getArgStringValue)("importPath", directive);
  const directory = import_path.default.join(import_path.default.dirname(outputFile), filename);
  const { memorySize, timeout } = import_config.HandlerConfigSchema.parse({
    ...config.handlerDefaults,
    ...(0, import_helpers.getOptionalArgObjectValue)("handlerConfig", directive)
  });
  const nestedStackLocation = "./cdc.yml";
  return {
    actionsModuleId: (0, import_paths.resolveActionsModuleId)(config, directory),
    directory,
    handlerImportName,
    handlerModuleId: (0, import_paths.resolveHandlerModuleId)(type, directory, handler),
    memorySize,
    nestedStackLocation,
    runtimeModuleId: "@code-like-a-carpenter/foundation-runtime",
    sourceModelName,
    timeout
  };
}
function extractEnricherConfig(config, schema, type, directive, outputFile) {
  const event = getEvent(type, directive);
  const sourceModelName = type.name;
  const targetModelName = (0, import_helpers.getArgStringValue)("targetModel", directive);
  const filename = `enricher--${(0, import_kebabCase.default)(
    sourceModelName
  )}--${event.toLowerCase()}--${(0, import_kebabCase.default)(targetModelName)}`;
  const functionName = makeFunctionName(
    "enricher",
    sourceModelName,
    event,
    targetModelName
  );
  const readableTables = getTargetTables("readableModels", schema, directive);
  const writableTables = Array.from(
    /* @__PURE__ */ new Set([
      ...getTargetTables("writableModels", schema, directive),
      getTargetTable(schema, type.name, targetModelName)
    ])
  );
  return {
    ...extractCommonConfig(
      config,
      schema,
      type,
      directive,
      outputFile,
      filename
    ),
    event,
    filename,
    functionName,
    readableTables,
    targetModelName,
    type: "ENRICHER",
    writableTables
  };
}
function extractReactorConfig(config, schema, type, directive, outputFile) {
  const event = getEvent(type, directive);
  const readableTables = getTargetTables("readableModels", schema, directive);
  const writableTables = getTargetTables("writableModels", schema, directive);
  const sourceModelName = type.name;
  const filename = `react--${(0, import_kebabCase.default)(
    sourceModelName
  )}--${event.toLowerCase()}`;
  const functionName = makeFunctionName("react", sourceModelName, event);
  return {
    ...extractCommonConfig(
      config,
      schema,
      type,
      directive,
      outputFile,
      filename
    ),
    event,
    filename,
    functionName,
    readableTables,
    sourceModelName: type.name,
    type: "REACTOR",
    writableTables
  };
}
function extractReducerConfig(config, schema, type, directive, outputFile) {
  const event = getEvent(type, directive);
  const sourceModelName = type.name;
  const targetModelName = (0, import_helpers.getArgStringValue)("targetModel", directive);
  const filename = `reduce--${(0, import_kebabCase.default)(
    sourceModelName
  )}--${event.toLowerCase()}--${(0, import_kebabCase.default)(targetModelName)}`;
  const functionName = makeFunctionName(
    "reduce",
    sourceModelName,
    event,
    targetModelName
  );
  const readableTables = Array.from(
    /* @__PURE__ */ new Set([
      ...getTargetTables("readableModels", schema, directive),
      (0, import_tables.extractTableName)((0, import_graphql.assertObjectType)(type))
    ])
  );
  const writableTables = Array.from(
    /* @__PURE__ */ new Set([
      ...getTargetTables("writableModels", schema, directive),
      getTargetTable(schema, type.name, targetModelName)
    ])
  );
  return {
    ...extractCommonConfig(
      config,
      schema,
      type,
      directive,
      outputFile,
      filename
    ),
    event,
    filename,
    functionName,
    multiReduce: (0, import_helpers.getOptionalArgBooleanValue)("multiReduce", directive) ?? false,
    readableTables,
    sourceModelName: type.name,
    targetModelName,
    type: "REDUCER",
    writableTables
  };
}
function getEvent(type, directive) {
  const event = (0, import_helpers.getArgEnumValue)("event", directive);
  (0, import_assert.default)(
    event === "INSERT" || event === "MODIFY" || event === "REMOVE" || event === "UPSERT",
    `Invalid event type ${event} for @${directive.name.value} on ${type.name}`
  );
  return event;
}
function makeFunctionName(prefix, sourceModelName, event, targetModelName = "") {
  const sourceAbbr = abbreviate(sourceModelName);
  const targetAbbr = abbreviate(targetModelName);
  const rand = import_crypto.default.createHash("sha1").update(sourceModelName + event + targetModelName).digest("hex").slice(0, 8);
  const functionName = `Fn${(0, import_upperFirst.default)(
    (0, import_camelCase.default)(
      [prefix, sourceAbbr, event, targetAbbr].filter(import_helpers.filterNull).join("--")
    )
  )}${rand}`;
  (0, import_assert.default)(
    functionName.length <= 64,
    `Handler function name must be less than 64 characters: ${functionName}`
  );
  return functionName;
}
function abbreviate(str) {
  return (0, import_snakeCase.default)(str).split("_").map((c) => c[0]).join("-");
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  extractChangeDataCaptureConfig,
  extractDispatcherConfig,
  getTargetTables
});
//# sourceMappingURL=cdc.cjs.map
