"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// packages/@code-like-a-carpenter/foundation-parser/src/models.ts
var models_exports = {};
__export(models_exports, {
  extractModel: () => extractModel,
  getModel: () => getModel
});
module.exports = __toCommonJS(models_exports);
var import_assert = __toESM(require("assert"));
var import_graphql = require("graphql");
var import_camelCase = __toESM(require("lodash/camelCase.js"));
var import_snakeCase = __toESM(require("lodash/snakeCase.js"));
var import_cdc = require("./extractors/cdc.cjs");
var import_indexes = require("./extractors/indexes.cjs");
var import_primary_key = require("./extractors/primary-key.cjs");
var import_ttl = require("./extractors/ttl.cjs");
var import_helpers = require("./helpers.cjs");
var import_tables = require("./tables.cjs");
var models = /* @__PURE__ */ new Map();
function getModel(type) {
  const model = models.get(type);
  (0, import_assert.default)(model, `Model for type ${type} not found`);
  return model;
}
function extractModel(config, schema, type, outputPath) {
  const cachedModel = models.get(type);
  if (cachedModel) {
    return cachedModel;
  }
  const fields = extractFields(type);
  const fieldMap = Object.fromEntries(
    fields.map((field) => [field.fieldName, field])
  );
  const model = {
    changeDataCaptureConfig: (0, import_cdc.extractChangeDataCaptureConfig)(
      config,
      schema,
      type,
      outputPath
    ),
    consistent: (0, import_helpers.hasDirective)("consistent", type),
    fields,
    isLedger: (0, import_helpers.hasDirective)("ledger", type),
    isPublic: (0, import_helpers.hasInterface)("PublicModel", type),
    primaryKey: (0, import_primary_key.extractPrimaryKey)(type, fieldMap),
    secondaryIndexes: (0, import_indexes.extractSecondaryIndexes)(config, type, fieldMap),
    ttlConfig: (0, import_ttl.extractTTLConfig)(type),
    typeName: type.name,
    ...extractTableInfo(type)
  };
  models.set(type, model);
  const wholeModel = model;
  const table = (0, import_tables.extractTable)(config, schema, type, outputPath);
  wholeModel.table = table;
  return wholeModel;
}
function extractFields(type) {
  const fields = type.getFields();
  return Object.keys(fields).map((fieldName) => {
    const field = fields[fieldName];
    const computed = (0, import_helpers.getOptionalDirective)("computed", field);
    const importDetails = computed ? {
      importName: (0, import_helpers.getArgStringValue)("importName", computed),
      importPath: (0, import_helpers.getArgStringValue)("importPath", computed),
      isVirtual: !!(0, import_helpers.getOptionalArgBooleanValue)("virtual", computed)
    } : void 0;
    return {
      columnName: getAliasForField(field, type, fieldName),
      columnNamesForRead: getReadAliasesForField(field, type, fieldName),
      computeFunction: importDetails,
      ean: `:${fieldName}`,
      eav: `#${fieldName}`,
      fieldName,
      isDateType: (0, import_helpers.isType)("Date", field),
      isRequired: (0, import_graphql.isNonNullType)(field.type),
      isScalarType: (0, import_graphql.isNonNullType)(field.type) ? (0, import_graphql.isScalarType)(field.type.ofType) : (0, import_graphql.isScalarType)(field.type),
      typeName: (0, import_graphql.isNonNullType)(field.type) ? String(field.type.ofType) : field.type.name
    };
  });
}
function extractTableInfo(type) {
  const tableDirective = (0, import_helpers.getOptionalDirective)("table", type);
  return {
    enablePointInTimeRecovery: tableDirective ? (0, import_helpers.getOptionalArgBooleanValue)(
      "enablePointInTimeRecovery",
      tableDirective
    ) !== false : true
  };
}
function getAliasForField(field, type, fieldName) {
  if ((0, import_helpers.hasDirective)("ttl", field)) {
    return "ttl";
  }
  if ((0, import_helpers.hasDirective)("alias", field)) {
    const { astNode } = field;
    (0, import_assert.default)(astNode);
    return (0, import_helpers.getArgStringValue)("name", (0, import_helpers.getDirective)("alias", astNode));
  }
  switch (field.name) {
    case "version":
      return "_v";
    case "createdAt":
      return "_ct";
    case "updatedAt":
      return "_md";
    case "publicId":
      return "publicId";
    default:
      return getCaseType(type) === "CAMEL_CASE" ? (0, import_camelCase.default)(fieldName) : (0, import_snakeCase.default)(fieldName);
  }
}
function getReadAliasesForField(field, type, fieldName) {
  if ((0, import_helpers.hasDirective)("ttl", field)) {
    return ["ttl"];
  }
  if ((0, import_helpers.hasDirective)("alias", field)) {
    const { astNode } = field;
    (0, import_assert.default)(astNode);
    return [(0, import_helpers.getArgStringValue)("name", (0, import_helpers.getDirective)("alias", astNode))];
  }
  switch (field.name) {
    case "version":
      return ["_v"];
    case "createdAt":
      return ["_ct"];
    case "updatedAt":
      return ["_md"];
    case "publicId":
      return ["publicId"];
    default:
      return getCaseType(type) === "CAMEL_CASE" ? [(0, import_camelCase.default)(fieldName), (0, import_snakeCase.default)(fieldName)] : [(0, import_snakeCase.default)(fieldName), (0, import_camelCase.default)(fieldName)];
  }
}
function getCaseType(type) {
  const tableDirective = (0, import_helpers.getOptionalDirective)("table", type);
  if (tableDirective) {
    const arg = (0, import_helpers.getOptionalArg)("columnCase", tableDirective);
    if (arg) {
      (0, import_assert.default)(arg.value.kind === "EnumValue");
      if (arg.value.value === "CAMEL_CASE") {
        return "CAMEL_CASE";
      }
      (0, import_assert.default)(arg.value.value === "SNAKE_CASE");
    }
  }
  return "SNAKE_CASE";
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  extractModel,
  getModel
});
//# sourceMappingURL=models.cjs.map
