import type { ConstDirectiveNode, FieldDefinitionNode, GraphQLField, GraphQLObjectType, ObjectTypeDefinitionNode } from 'graphql';
/** Gets the specified argument from the given directive. */
export declare function getArg(name: string, directive: ConstDirectiveNode): import("graphql").ConstArgumentNode;
/** Gets the specified argument from the given directive. */
export declare function getOptionalArg(name: string, directive: ConstDirectiveNode): import("graphql").ConstArgumentNode | undefined;
/**
 * Gets the string value of the specified argument from the given directive.
 */
export declare function getArgStringValue(fieldName: string, directive: ConstDirectiveNode): string;
/**
 * Gets the enum value of the specified argument from the given directive as a
 * string
 */
export declare function getArgEnumValue(fieldName: string, directive: ConstDirectiveNode): string;
/**
 * Given a field name that identifies a list argument, returns the typescript
 * types identified by those strings.
 */
export declare function getArgStringArrayValue(fieldName: string, directive: ConstDirectiveNode): string[];
/**
 * Gets the boolean value of the specified argument from the given directive.
 * Returns undefined
 */
export declare function getOptionalArgBooleanValue(fieldName: string, directive: ConstDirectiveNode): boolean | undefined;
/**
 * Given a field name that identifies a list argument, returns the typescript
 * types identified by those strings.
 */
export declare function getOptionalArgStringArrayValue(fieldName: string, directive: ConstDirectiveNode): string[] | undefined;
/**
 * Gets the string value of the specified argument from the given directive.
 * Returns an empty string if the argument is not present.
 */
export declare function getOptionalArgStringValue(fieldName: string, directive: ConstDirectiveNode): string;
/** Gets the specified directive from the given field. */
export declare function getDirective(name: string, nodeOrType: FieldDefinitionNode | ObjectTypeDefinitionNode | GraphQLObjectType): ConstDirectiveNode;
/** Gets the specified directive from the given field. */
export declare function getOptionalDirective(name: string, nodeOrType: FieldDefinitionNode | ObjectTypeDefinitionNode | GraphQLObjectType | GraphQLField<unknown, unknown>): ConstDirectiveNode | undefined;
/** Indicates if objType contains the specified directive */
export declare function hasDirective(directiveName: string, objType: GraphQLObjectType | GraphQLField<unknown, unknown>): boolean;
/** Indicates if objType implements the specified interface */
export declare function hasInterface(interfaceName: string, objType: GraphQLObjectType): boolean;
/**
 * Indicates if field is the specified type. Does not care if field is NonNull
 */
export declare function isType(typeName: string, fieldType: GraphQLField<unknown, unknown>): boolean;
/** Typesafe function for .filter to remove undefined or null values */
export declare function filterNull<T>(x: T | undefined | null): x is T;
/**
 * Returns all the scalar arguments nested on the given directive's field.
 */
export declare function getOptionalArgObjectValue(fieldName: string, directive: ConstDirectiveNode): Record<string, unknown> | undefined;
//# sourceMappingURL=helpers.d.ts.map